/* gInfo - Copyright (C) 2004-5 bakunin - Andrea Marchesini 
 *                                        <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <editor.h>

static GtkWidget *preferences_spaces_cb;
static GtkWidget *preferences_autoi_cb;
static GtkWidget *preferences_tab_spin;
static GtkWidget *preferences_undol_rb;
static GtkWidget *preferences_undol_spin;
static GtkWidget *preferences_undou_rb;

static int preferences_tab_width = 8;
static int preferences_tab_spaces = 0;
static int preferences_auto_indent = 1;
static int preferences_undo = 1;
static int preferences_undo_n = 25;

void preferences_close (GtkWidget *, gpointer);
void preferences_undo_changed (GtkWidget *, gpointer);
void preferences_ok (GtkWidget *, gpointer);
void preferences_save (void);
int preferences_init (void);

/* Show the preferences window */
void
preferences_create (void)
{
  GtkWidget *prefwin;
  GtkWidget *frame;
  GtkWidget *label;
  GtkObject *spin_adj;
  GtkWidget *box;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *button;
  GSList *radiobutton_group = NULL;

  char buf[1024];

  snprintf (buf, 1024, "%s %s - preferences", NAME, VERSION);

  prefwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (prefwin), buf);

  g_signal_connect ((gpointer) prefwin, "destroy",
		    G_CALLBACK (gtk_widget_destroy), NULL);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (prefwin), vbox);

  frame = gtk_frame_new ("Preferences");
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);

  box = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (box);
  gtk_container_add (GTK_CONTAINER (frame), box);

  label = gtk_label_new ("<b>Tabs</b>");
  gtk_widget_show (label);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_misc_set_padding (GTK_MISC (label), 0, 10);
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);

  label = gtk_label_new ("Tabs");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

  spin_adj = gtk_adjustment_new (preferences_tab_width, 0, 20, 1, 5, 10);
  preferences_tab_spin =
    gtk_spin_button_new (GTK_ADJUSTMENT (spin_adj), 1, 0);
  gtk_widget_show (preferences_tab_spin);
  gtk_box_pack_start (GTK_BOX (hbox), preferences_tab_spin, FALSE, FALSE, 0);

  preferences_spaces_cb =
    gtk_check_button_new_with_mnemonic ("_Insert spaces instead of tabs");
  gtk_widget_show (preferences_spaces_cb);
  gtk_box_pack_start (GTK_BOX (box), preferences_spaces_cb, FALSE, FALSE, 0);

  if (preferences_tab_spaces)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (preferences_spaces_cb),
				  TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (preferences_spaces_cb),
				  FALSE);

  label = gtk_label_new ("<b>Auto Indentation</b>");
  gtk_widget_show (label);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_misc_set_padding (GTK_MISC (label), 0, 10);
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);

  preferences_autoi_cb =
    gtk_check_button_new_with_mnemonic ("_Enable auto indentation");
  gtk_widget_show (preferences_autoi_cb);
  gtk_box_pack_start (GTK_BOX (box), preferences_autoi_cb, FALSE, FALSE, 0);

  if (preferences_auto_indent)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (preferences_autoi_cb),
				  TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (preferences_autoi_cb),
				  FALSE);

  label = gtk_label_new ("<b>Undo</b>");
  gtk_widget_show (label);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_misc_set_padding (GTK_MISC (label), 0, 10);
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);

  preferences_undol_rb =
    gtk_radio_button_new_with_mnemonic (radiobutton_group, "_Limit to ");
  gtk_widget_show (preferences_undol_rb);
  gtk_box_pack_start (GTK_BOX (hbox), preferences_undol_rb, FALSE, FALSE, 0);
  radiobutton_group =
    gtk_radio_button_get_group (GTK_RADIO_BUTTON (preferences_undol_rb));

  g_signal_connect ((gpointer) preferences_undol_rb, "toggled",
		    G_CALLBACK (preferences_undo_changed), NULL);

  spin_adj = gtk_adjustment_new (preferences_undo_n, 0, 200, 1, 5, 10);
  preferences_undol_spin =
    gtk_spin_button_new (GTK_ADJUSTMENT (spin_adj), 1, 0);
  gtk_widget_show (preferences_undol_spin);
  gtk_box_pack_start (GTK_BOX (hbox), preferences_undol_spin, FALSE, FALSE,
		      0);

  label = gtk_label_new (" actions");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

  preferences_undou_rb =
    gtk_radio_button_new_with_mnemonic (radiobutton_group, "_Unlimited ");
  gtk_widget_show (preferences_undou_rb);
  gtk_box_pack_start (GTK_BOX (box), preferences_undou_rb, FALSE, FALSE, 0);
  radiobutton_group =
    gtk_radio_button_get_group (GTK_RADIO_BUTTON (preferences_undou_rb));

  if (preferences_undo)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (preferences_undol_rb),
				  TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (preferences_undou_rb),
				  TRUE);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);

  button = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);

  g_signal_connect ((gpointer) button, "clicked",
		    G_CALLBACK (preferences_close), prefwin);

  button = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);

  g_signal_connect ((gpointer) button, "clicked", G_CALLBACK (preferences_ok),
		    prefwin);

  gtk_widget_show (prefwin);
}

/* Close the preferences window */
void
preferences_close (GtkWidget * w, gpointer dummy)
{
  gtk_widget_destroy ((GtkWidget *) dummy);
}

/* set sensitive or not the undo flags */
void
preferences_undo_changed (GtkWidget * w, gpointer dummy)
{
  if (!gtk_toggle_button_get_active
      (GTK_TOGGLE_BUTTON (preferences_undol_rb)))
    gtk_widget_set_sensitive (preferences_undol_spin, FALSE);
  else
    gtk_widget_set_sensitive (preferences_undol_spin, TRUE);

}

/* the the new preference, save it and set it */
void
preferences_ok (GtkWidget * w, gpointer dummy)
{
  int tab, spaces, indent, limit, limit_n;

  tab =
    (int) gtk_spin_button_get_value (GTK_SPIN_BUTTON (preferences_tab_spin));
  spaces =
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (preferences_spaces_cb));
  indent =
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (preferences_autoi_cb));
  limit =
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (preferences_undol_rb));
  limit_n =
    (int)
    gtk_spin_button_get_value (GTK_SPIN_BUTTON (preferences_undol_spin));

  gtk_widget_destroy ((GtkWidget *) dummy);

  editor_set_preferences (tab, spaces, indent, limit, limit_n);

  preferences_tab_width = tab;
  preferences_tab_spaces = spaces;
  preferences_auto_indent = indent;
  preferences_undo = limit;
  preferences_undo_n = limit_n;

  preferences_save ();
}

/* Save the new preferences in ~/.ginfo-editor.cfg */
void
preferences_save (void)
{
  int n;
  FILE *fl;
  char *home, *t;

  if (!(home = getenv ("HOME")))
    {
      editor_dialog
	("No HOME enviroment. I can't initialize your configuration.");
      return;
    }

  n = strlen (home) + 20;
  if (!(t = (char *) malloc (sizeof (char *) * n)))
    fatal ("Error: memory.");

  snprintf (t, n, "%s/.ginfo-editor.cfg", home);

  if (!(fl = fopen (t, "w")))
    {
      editor_dialog ("I can't write your configuration file: %s", t);
      return;
    }

  fprintf (fl, "tab_width = %d\n", preferences_tab_width);
  fprintf (fl, "tab_spaces = %s\n",
	   preferences_tab_spaces ? "true" : "false");
  fprintf (fl, "auto_indent = %s\n",
	   preferences_auto_indent ? "true" : "false");
  fprintf (fl, "undo = %s\n", preferences_undo ? "true" : "false");
  fprintf (fl, "undo_n = %d\n", preferences_undo_n);

  fclose (fl);
  free (t);

  return;
}

/* Read  ~/.ginfo-editor.cfg and restore the preferences */
void
preferences_open (void)
{
  char *home;
  char *t;
  int n;
  cfg_t *cfg;

  cfg_opt_t opts[] = {
    CFG_INT ("tab_width", 8, CFGF_NONE),
    CFG_BOOL ("tab_spaces", cfg_false, CFGF_NONE),
    CFG_BOOL ("auto_indent", cfg_true, CFGF_NONE),
    CFG_BOOL ("undo", cfg_true, CFGF_NONE),
    CFG_INT ("undo_n", 25, CFGF_NONE),
    CFG_END ()
  };

  if (preferences_init ())
    return;

  if (!(home = getenv ("HOME")))
    return;

  n = strlen (home) + 20;
  if (!(t = (char *) malloc (sizeof (char) * n)))
    fatal ("Error: memory.");

  snprintf (t, n, "%s/.ginfo-editor.cfg", home);

  cfg = cfg_init (opts, CFGF_NOCASE);

  switch (cfg_parse (cfg, t))
    {
    case CFG_FILE_ERROR:
      free (t);
      return;
    case CFG_PARSE_ERROR:
      unlink (t);
      free (t);
      return;
    }

  preferences_tab_width = cfg_getint (cfg, "tab_width");
  preferences_tab_spaces = cfg_getbool (cfg, "tab_spaces");
  preferences_auto_indent = cfg_getbool (cfg, "auto_indent");
  preferences_undo = cfg_getbool (cfg, "undo");
  preferences_undo_n = cfg_getint (cfg, "undo_n");

  editor_set_preferences (cfg_getint (cfg, "tab_width"),
			  cfg_getbool (cfg, "tab_spaces"),
			  cfg_getbool (cfg, "auto_indent"),
			  cfg_getbool (cfg, "undo"),
			  cfg_getint (cfg, "undo_n"));

  cfg_free (cfg);
  free (t);
}

/* Inizialize the ~/.ginfo-editor.cfg file */
int
preferences_init (void)
{
  char *home;
  char *t;
  int n;
  struct stat st;
  FILE *fl;

  if (!(home = getenv ("HOME")))
    return 1;

  n = strlen (home) + 20;

  if (!(t = (char *) malloc (sizeof (char) * n)))
    fatal ("Error: memory.");

  snprintf (t, n, "%s/.ginfo-editor.cfg", home);

  if (lstat (t, &st))
    {
      if (!(fl = fopen (t, "w")))
	{
	  free (t);
	  return 1;
	}

      fprintf (fl, "tab_width = 8\n");
      fprintf (fl, "tab_spaces = false\n");
      fprintf (fl, "auto_indent = true\n");
      fprintf (fl, "undo = true\n");
      fprintf (fl, "undo_n = 25\n");

      fclose (fl);
    }

  free (t);

  return 0;
}

/* EOF */
