/* gInfo - Copyright (C) 2004-5 bakunin - Andrea Marchesini 
 *                                        <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <editor.h>

char *print_title (char *);
void print_dialog_response (GtkWidget *, int, GtkSourcePrintJob *);

void
print_preview (GtkWidget * w, gpointer dummy)
{
  GtkSourcePrintJob *job;
  GnomePrintJob *print_job;
  GtkWidget *preview;

  char *title = print_title (editor_output);

  job = gtk_source_print_job_new_with_buffer (NULL, sourcebuffer);
  gtk_source_print_job_set_header_footer_font (job, "Sans Regular 12.0");
  gtk_source_print_job_set_header_format (job, NULL, title, NULL, TRUE);
  gtk_source_print_job_set_print_header (job, TRUE);
  gtk_source_print_job_set_footer_format (job, NULL, NULL, "Page %N of %Q",
					  TRUE);
  gtk_source_print_job_set_print_footer (job, TRUE);

  print_job = gtk_source_print_job_print (job);
  preview = (GtkWidget *) gnome_print_job_preview_new (print_job, title);
  gtk_widget_show (preview);

  g_object_unref (job);
  free (title);
}

void
print (GtkWidget * w, gpointer dummy)
{
  GtkWidget *dialog;
  GtkSourcePrintJob *job;
  GnomePrintConfig *print_config;

  char *title = print_title (editor_output);

  job = gtk_source_print_job_new_with_buffer (NULL, sourcebuffer);
  gtk_source_print_job_set_header_footer_font (job, "Sans Regular 12.0");
  gtk_source_print_job_set_header_format (job, NULL, title, NULL, TRUE);
  gtk_source_print_job_set_print_header (job, TRUE);
  gtk_source_print_job_set_footer_format (job, NULL, NULL, "Page %N of %Q",
					  TRUE);
  gtk_source_print_job_set_print_footer (job, TRUE);

  print_config = gtk_source_print_job_get_config (job);

  dialog =
    g_object_new (GNOME_TYPE_PRINT_DIALOG, "print_config", print_config,
		  NULL);
  gnome_print_dialog_construct (GNOME_PRINT_DIALOG (dialog), "Print",
				GNOME_PRINT_DIALOG_RANGE |
				GNOME_PRINT_DIALOG_COPIES);

  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);


  g_signal_connect (dialog, "response", G_CALLBACK (print_dialog_response),
		    job);

  gtk_widget_show (dialog);
  free (title);
}

void
print_dialog_response (GtkWidget * dialog, int response,
		       GtkSourcePrintJob * job)
{
  GnomePrintJob *print_job;

  switch (response)
    {
    case GNOME_PRINT_DIALOG_RESPONSE_PRINT:
      print_job = gtk_source_print_job_print (job);
      gnome_print_job_print (print_job);
      break;

    case GNOME_PRINT_DIALOG_RESPONSE_PREVIEW:
      print_preview (NULL, NULL);
      break;
    }

  gtk_widget_destroy (dialog);
  g_object_unref (job);
}

char *
print_title (char *t)
{
  int i;
  char *o;

  if (!t)
    {
      if (!(o = strdup ("Unknow")))
	fatal ("Error: memory.");
      return o;
    }

  i = strlen (t) - 1;
  while (i)
    {
      if (t[i] == '/')
	break;
      i--;
    }

  if (i)
    i++;
  if (!(o = strdup (t + i)))
    fatal ("Error: memory.");

  return o;
}

/* EOF */
