#include <ginfo_plugin.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <dirent.h>
#include <dlfcn.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <libxml/parser.h>
#include <libgtkhtml/gtkhtml.h>

struct html2ginfo_queue
{
  char *url;
  struct html2ginfo_queue *next;
  struct html2ginfo_queue *prev;
};

struct html2ginfo_queue *html2ginfo_queue_start = NULL;
struct html2ginfo_queue *html2ginfo_queue_current = NULL;

GtkWidget *create_html (void *);
void html2ginfo_link (HtmlDocument *, const gchar *, gpointer);
gboolean html2ginfo_url (HtmlDocument *, gchar *, HtmlStream *, gpointer);

GtkWidget *html2ginfo_title;
GtkWidget *html2ginfo_back;
GtkWidget *html2ginfo_forward;

GINFO_INIT;

#ifdef PHP
char *
html2ginfo_php_file_create (void)
{
  static char buf[1024];

  snprintf (buf, sizeof (buf), "%s/html2ginfo_%d.php", g_get_tmp_dir (),
	    getpid ());

  return buf;
}

void
html2ginfo_php_file_delete (void)
{
  char buf[1024];

  snprintf (buf, sizeof (buf), "%s/html2ginfo_%d.php", g_get_tmp_dir (),
	    getpid ());

  unlink (buf);
}

int
html2ginfo_open_php (char *f)
{
  FILE *php;
  char buf[1024];
  int ret;
  char *file;
  FILE *fd;

  snprintf (buf, sizeof (buf), "%s %s", PHP, f);

  if (!(php = popen (buf, "r")))
    ginfo_error ("I cant run PHP interpeter");

  file = html2ginfo_php_file_create ();

  if (!(fd = fopen (file, "w")))
    ginfo_error ("I can't create the temp file.");

  while (!feof (php))
    {
      ret = fread (buf, 1, sizeof (buf), php);
      fwrite (buf, 1, ret, fd);
    }

  fclose (fd);

  pclose (php);

  return open (file, O_RDONLY);
}
#endif

void
html2ginfo_queue_check (void)
{
  if (!html2ginfo_queue_start || !html2ginfo_queue_current->prev)
    gtk_widget_set_sensitive (html2ginfo_back, FALSE);
  else
    gtk_widget_set_sensitive (html2ginfo_back, TRUE);

  if (!html2ginfo_queue_start || !html2ginfo_queue_current->next)
    gtk_widget_set_sensitive (html2ginfo_forward, FALSE);
  else
    gtk_widget_set_sensitive (html2ginfo_forward, TRUE);
}

void
html2ginfo_queue_free (struct html2ginfo_queue *a)
{
  struct html2ginfo_queue *old;

  while (a)
    {
      old = a;

      free (a->url);
      a = a->next;
      free (old);
    }
}

void
html2ginfo_queue_add (char *url)
{
  struct html2ginfo_queue *a;

  if (!html2ginfo_queue_start)
    {
      if (!
	  (a =
	   (struct html2ginfo_queue *)
	   malloc (sizeof (struct html2ginfo_queue))))
	ginfo_error ("Memory!");

      html2ginfo_queue_current = html2ginfo_queue_start = a;
      a->next = NULL;
      a->prev = NULL;

      a->url = strdup (url);
    }
  else
    {

      if (!
	  (a =
	   (struct html2ginfo_queue *)
	   malloc (sizeof (struct html2ginfo_queue))))
	ginfo_error ("Memory!");

      html2ginfo_queue_free (html2ginfo_queue_current->next);

      html2ginfo_queue_current->next = a;

      a->url = strdup (url);
      a->next = NULL;
      a->prev = html2ginfo_queue_current;
      html2ginfo_queue_current = a;
    }
  html2ginfo_queue_check ();
}

void
html2ginfo_queue_back (GtkWidget * w, gpointer dummy)
{

  if (!html2ginfo_queue_current || !html2ginfo_queue_current->prev)
    return;

  html2ginfo_queue_current = html2ginfo_queue_current->prev;

  html2ginfo_link ((HtmlDocument *) dummy, html2ginfo_queue_current->url, 1);

  html2ginfo_queue_check ();
}

void
html2ginfo_queue_forward (GtkWidget * w, gpointer dummy)
{

  if (!html2ginfo_queue_current || !html2ginfo_queue_current->next)
    return;

  html2ginfo_queue_current = html2ginfo_queue_current->next;

  html2ginfo_link ((HtmlDocument *) dummy, html2ginfo_queue_current->url, 1);
  html2ginfo_queue_check ();
}

void
html2ginfo_404 (HtmlDocument * doc, char *url)
{
  gchar *msg = g_strdup_printf
    ("<html>"
     "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />"
     "<head><title>Opsss... Page not found!</title></head>"
     "<body bgcolor=\"white\">"
     "<div align=\"center\">"
     "<h1>Opsss... Page not found!</h1><br>"
     "<h3>The requested URL could not be loaded:</h3>"
     "<tt>%s</tt>" "</div>" "</body>" "</html>",
     url);

  html_document_write_stream (doc, msg, strlen (msg));

  g_free (msg);
}

char *
html2ginfo_path (char *url)
{
  static char buf[1024];

  if (url[0] == '/')
    return url;
  snprintf (buf, sizeof (buf), "%s/%s", HTML_DIR, url);

  return buf;
}

void
html2ginfo_link (HtmlDocument * doc, const gchar * url, gpointer data)
{
  html_document_clear (doc);
  html_document_open_stream (doc, "text/html");

  if (html2ginfo_url
      (doc, html2ginfo_path ((char *) url), doc->current_stream,
       NULL) == FALSE)
    {
      html2ginfo_404 (doc, (char *) url);
    }

  else if (!data)
    html2ginfo_queue_add (html2ginfo_path ((char *) url));

}

gboolean
io_handler (GIOChannel * io, GIOCondition condition, gpointer data)
{
  HtmlStream *stream = data;
  gchar buffer[8192];
  gsize bytes;

  if (condition & G_IO_IN)
    {
      if (g_io_channel_read_chars (io, buffer, sizeof (buffer),
				   &bytes, NULL) != G_IO_STATUS_ERROR
	  && bytes > 0)
	{
	  html_stream_write (stream, buffer, bytes);
	}
      else
	{
	  html_stream_close (stream);
	  g_io_channel_unref (io);

	  return FALSE;
	}

      if (condition & G_IO_HUP)
	{
	  while (g_io_channel_read_chars (io, buffer, sizeof (buffer),
					  &bytes, NULL) != G_IO_STATUS_ERROR
		 && bytes > 0)
	    {
	      html_stream_write (stream, buffer, bytes);
	    }
	}
    }

  if (condition & (G_IO_ERR | G_IO_HUP | G_IO_NVAL))
    {
      html_stream_close (stream);
      g_io_channel_unref (io);

      return FALSE;
    }

  return TRUE;
}


gboolean
html2ginfo_url (HtmlDocument * doc, gchar * url, HtmlStream * stream,
		gpointer dummy)
{
  struct stat st;
  gint fd;

  g_return_val_if_fail (url != NULL, TRUE);
  g_return_val_if_fail (stream != NULL, TRUE);

  url = html2ginfo_path (url);

  if (lstat (url, &st))
    return FALSE;

#ifdef PHP
  if (strcmp (url + (strlen (url) - 4), ".php"))
#endif
    fd = open (url, O_RDONLY);
#ifdef PHP
  else
    fd = html2ginfo_open_php (url);
#endif

  if (fd < 0)
    return FALSE;

  GIOChannel *io = g_io_channel_unix_new (fd);

  g_io_channel_set_close_on_unref (io, TRUE);
  g_io_channel_set_encoding (io, NULL, NULL);

  g_io_add_watch (io, G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
		  io_handler, stream);

#ifdef PHP
  if (!strcmp (url + (strlen (url) - 4), ".php"))
    html2ginfo_php_file_delete ();
#endif

  return TRUE;
}

void
html2ginfo_set_title (HtmlDocument * doc,
		      const gchar * new_title, gpointer data)
{
  if (new_title)
    gtk_label_set_text (GTK_LABEL (html2ginfo_title), new_title);
  else
    gtk_label_set_text (GTK_LABEL (html2ginfo_title), "Untitled");
}

GtkWidget *
create_html (void *arg)
{

  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *sw;
  HtmlDocument *doc;
  GtkWidget *html;
  GtkWidget *box;
  GtkWidget *label;
  GtkWidget *image;

  vbox = gtk_vbox_new (FALSE, 0);
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);

  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  doc = html_document_new ();
  g_signal_connect (G_OBJECT (doc), "request_url",
		    G_CALLBACK (html2ginfo_url), NULL);
  g_signal_connect (G_OBJECT (doc), "link_clicked",
		    G_CALLBACK (html2ginfo_link), NULL);
  g_signal_connect (G_OBJECT (doc), "title_changed",
		    G_CALLBACK (html2ginfo_set_title), NULL);

  html_document_open_stream (doc, "text/html");

  if (html2ginfo_url
      (doc, html2ginfo_path ((char *) arg), doc->current_stream,
       NULL) == FALSE)
    html2ginfo_404 (doc, (char *) arg);

  html = html_view_new ();

  html_view_set_document (HTML_VIEW (html), doc);

  gtk_container_add (GTK_CONTAINER (sw), html);
  gtk_box_pack_start (GTK_BOX (vbox), sw, TRUE, TRUE, 0);

  html2ginfo_back = gtk_button_new ();
  gtk_widget_show (html2ginfo_back);
  gtk_box_pack_start (GTK_BOX (hbox), html2ginfo_back, FALSE, FALSE, 0);
  g_signal_connect ((gpointer) html2ginfo_back, "clicked",
		    G_CALLBACK (html2ginfo_queue_back), doc);

  box = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (box);
  gtk_container_add (GTK_CONTAINER (html2ginfo_back), box);

  image = gtk_image_new_from_stock ("gtk-go-back", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image);
  gtk_box_pack_start (GTK_BOX (box), image, TRUE, TRUE, 0);

  label = gtk_label_new ("Back");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);

  html2ginfo_title = gtk_label_new ("");
  gtk_widget_show (html2ginfo_title);
  gtk_box_pack_start (GTK_BOX (hbox), html2ginfo_title, TRUE, TRUE, 0);

  html2ginfo_forward = gtk_button_new ();
  gtk_widget_show (html2ginfo_forward);
  gtk_box_pack_start (GTK_BOX (hbox), html2ginfo_forward, FALSE, FALSE, 0);
  g_signal_connect ((gpointer) html2ginfo_forward, "clicked",
		    G_CALLBACK (html2ginfo_queue_forward), doc);

  box = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (box);
  gtk_container_add (GTK_CONTAINER (html2ginfo_forward), box);

  label = gtk_label_new ("Forward");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);

  image = gtk_image_new_from_stock ("gtk-go-forward", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image);
  gtk_box_pack_start (GTK_BOX (box), image, TRUE, TRUE, 0);

  html2ginfo_queue_add ((char *) arg);

  html2ginfo_queue_check ();
  return vbox;
}

int
module_default (void)
{
  return 0;
}

char *
module_name (void)
{
  return "html2ginfo";
}

int
module_genre (void)
{
  return GINFO_GENRE_SYSTEM;
}

char **
module_words (void)
{
  char **list;
  DIR *d;
  struct dirent *dir;
  int n = 0;

  if (!(d = opendir (HTML_DIR)))
    {
      puts ("No html founded...");
      return NULL;
    }

  while ((dir = readdir (d)) != NULL)
    {
      if (strcmp (dir->d_name + (strlen (dir->d_name) - 5), ".html")
	  && strcmp (dir->d_name + (strlen (dir->d_name) - 4), ".htm")
#ifdef PHP
	  && strcmp (dir->d_name + (strlen (dir->d_name) - 4), ".php")
#endif
	)
	continue;
      n++;
    }

  if (!(list = (char **) malloc (sizeof (char *) * (n + 2))))
    ginfo_error ("Error: memory.");

  closedir (d);

  if (!(d = opendir (HTML_DIR)))
    {
      free (list);
      puts ("No html founded...");
      return NULL;
    }

  n = 0;
  while ((dir = readdir (d)) != NULL)
    {
      if (strcmp (dir->d_name + (strlen (dir->d_name) - 5), ".html")
	  && strcmp (dir->d_name + (strlen (dir->d_name) - 4), ".htm")
#ifdef PHP
	  && strcmp (dir->d_name + (strlen (dir->d_name) - 4), ".php")
#endif
	)
	continue;

      list[n] = strdup (dir->d_name);
      n++;
    }

  closedir (d);

  list[n] = NULL;

  return list;
}

ginfo_docs *
module_docs (int *n)
{
  void *t;
  DIR *d;
  struct dirent *dir;
  char *buf;
  int i;
  char *path;

  if (!(d = opendir (HTML_DIR)))
    {
      puts ("No html founded...");
      return NULL;
    }

  t = ginfo_easy_docs_init ();

  while ((dir = readdir (d)) != NULL)
    {
      if (strcmp (dir->d_name + (strlen (dir->d_name) - 5), ".html")
	  && strcmp (dir->d_name + (strlen (dir->d_name) - 4), ".htm")
#ifdef PHP
	  && strcmp (dir->d_name + (strlen (dir->d_name) - 4), ".php")
#endif
	)
	continue;

      buf = strdup (dir->d_name);
      for (i = 0; i < strlen (buf); i++)
	if (buf[i] == '.')
	  {
	    buf[i] = 0;
	    break;
	  }

      if (!
	  (path =
	   (char *) malloc (sizeof (char) *
			    (strlen (HTML_DIR) + strlen (dir->d_name) + 2))))
	ginfo_error ("Memory!");
      sprintf (path, "%s/%s", HTML_DIR, dir->d_name);

      ginfo_easy_docs_add (t, strdup (buf), create_html, path, NULL);
      free (buf);
    }

  closedir (d);

  return ginfo_easy_docs_close (t, n);
}
