#include <ginfo_plugin.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <dirent.h>
#include <dlfcn.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <libxml/parser.h>
#include <curl/curl.h>
#include <curl/types.h>
#include <curl/easy.h>
#include <pthread.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <errno.h>

#define SIZE_100	1
#define SIZE_75		2
#define SIZE_50		3
#define SIZE_25		4
#define SIZE_125	5
#define SIZE_150	6
#define SIZE_200	7
#define SIZE_400	8
#define SIZE_AUTO	9
#define SIZE_WIDTH	10
#define SIZE_HEIGHT	11

void pdf2ginfo_check_pages (char *);
GtkWidget *pdf2ginfo_create_image (char *, int);
void pdf2ginfo_go (GtkWidget *, char *);
void pdf2ginfo_go_b (GtkWidget *, GtkWidget *);
void pdf2ginfo_go_page (char *, char *);
void pdf2ginfo_change_next (GtkWidget *, char *);
void pdf2ginfo_change_prev (GtkWidget *, char *);
GtkWidget *pdf2ginfo_popup_create (void);
GtkWidget *create_pdf (void *);
gint pdf2ginfo_popup (GtkWidget *, GdkEventButton *);

int pdf2ginfo_page;
char *pdf2ginfo_file;
int pdf2ginfo_scale;
GtkWidget *pdf2ginfo_image;
GtkWidget *pdf2ginfo_vp;
GtkWidget *pdf2ginfo_prev;
GtkWidget *pdf2ginfo_next;
GtkWidget *pdf2ginfo_label;
GtkWidget *pdf2ginfo_popup_m;
int pdf2ginfo_pages;

GINFO_INIT;

void
pdf2ginfo_check_pages (char *file)
{
  int fd;
  char pages[20];
  char *arg[3];
  int p[2];
  char buf[1024];
  int status;

  arg[0] = PDFINFO_PATH;
  arg[1] = file;
  arg[2] = NULL;

  pipe (p);

  fd = fork ();

  switch (fd)
    {
    case -1:
      pdf2ginfo_pages = 0;
      return;

    case 0:
      close (1);
      dup2 (p[1], 1);
      execv (arg[0], arg);
      exit (1);

    default:
      wait (&status);
      if (WEXITSTATUS (status))
	{
	  pdf2ginfo_pages = 0;
	  break;
	}

      fd = read (p[0], &buf, 1024);
      buf[fd] = 0;

      fd = 0;

      while (buf[fd])
	{
	  if (buf[fd] == '\n' && buf[++fd] == 'P' && buf[++fd] == 'a'
	      && buf[++fd] == 'g')
	    {
	      sscanf (buf + fd, "%s", pages);
	      fd += strlen (pages);
	      sscanf (buf + fd, "%s", pages);
	      pdf2ginfo_pages = atoi (pages);
	      break;

	    }
	  fd++;
	}
    }

  if (pdf2ginfo_pages > 0)
    {
      if (pdf2ginfo_page < pdf2ginfo_pages)
	gtk_widget_set_sensitive (pdf2ginfo_next, TRUE);
      else
	gtk_widget_set_sensitive (pdf2ginfo_next, FALSE);
    }

  if (pdf2ginfo_pages != 0 && pdf2ginfo_page != 1)
    gtk_widget_set_sensitive (pdf2ginfo_prev, TRUE);
  else
    gtk_widget_set_sensitive (pdf2ginfo_prev, FALSE);

  snprintf (buf, sizeof (buf), "Page: %d/%d", pdf2ginfo_page,
	    pdf2ginfo_pages);
  gtk_label_set_text (GTK_LABEL (pdf2ginfo_label), buf);
}

gint
pdf2ginfo_popup (GtkWidget * w, GdkEventButton * event)
{
  if (event->button == 3)
    {
      gtk_menu_popup (GTK_MENU (pdf2ginfo_popup_m), NULL, NULL, NULL, NULL,
		      event->button, event->time);
    }

  return FALSE;
}

GtkWidget *
pdf2ginfo_create_image (char *file, int page)
{
  char tmp[1024];
  GtkWidget *image;
  GdkPixbuf *i;
  int x, y;

  snprintf (tmp, sizeof (tmp), "%s -f %d -l %d %s %s/pdf2ginfo",
	    PDFTOPPM_PATH, page, page, file, g_get_tmp_dir ());

  if (system (tmp))
    ginfo_error ("pdf2ginfo: pdftoppm error!");

  snprintf (tmp, sizeof (tmp), "%s/pdf2ginfo-%.6d.ppm", g_get_tmp_dir (),
	    page);

  switch (pdf2ginfo_scale)
    {
    case SIZE_AUTO:
      ginfo_tutor_size (&x, &y);
      i = gdk_pixbuf_new_from_file_at_size (tmp, x - 50, y - 50, NULL);
      break;

    case SIZE_HEIGHT:
      ginfo_tutor_size (&x, &y);
      i = gdk_pixbuf_new_from_file (tmp, NULL);
      x = gdk_pixbuf_get_width (i);
      g_object_unref (i);
      i = gdk_pixbuf_new_from_file_at_size (tmp, x, y - 50, NULL);
      break;

    case SIZE_WIDTH:
      ginfo_tutor_size (&x, &y);
      i = gdk_pixbuf_new_from_file (tmp, NULL);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);
      i = gdk_pixbuf_new_from_file_at_size (tmp, x - 50, y, NULL);
      break;

    case SIZE_100:
      i = gdk_pixbuf_new_from_file (tmp, NULL);
      break;

    case SIZE_75:
      i = gdk_pixbuf_new_from_file (tmp, NULL);
      x = gdk_pixbuf_get_width (i);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);

      x *= 75;
      x /= 100;

      y *= 75;
      y /= 100;

      i = gdk_pixbuf_new_from_file_at_size (tmp, x, y, NULL);
      break;

    case SIZE_50:
      i = gdk_pixbuf_new_from_file (tmp, NULL);
      x = gdk_pixbuf_get_width (i);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);

      x *= 50;
      x /= 100;

      y *= 50;
      y /= 100;

      i = gdk_pixbuf_new_from_file_at_size (tmp, x, y, NULL);
      break;

    case SIZE_25:
      i = gdk_pixbuf_new_from_file (tmp, NULL);
      x = gdk_pixbuf_get_width (i);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);

      x *= 25;
      x /= 100;

      y *= 25;
      y /= 100;

      i = gdk_pixbuf_new_from_file_at_size (tmp, x, y, NULL);
      break;

    case SIZE_125:
      i = gdk_pixbuf_new_from_file (tmp, NULL);
      x = gdk_pixbuf_get_width (i);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);

      x *= 125;
      x /= 100;

      y *= 125;
      y /= 100;

      i = gdk_pixbuf_new_from_file_at_size (tmp, x, y, NULL);
      break;

    case SIZE_150:
      i = gdk_pixbuf_new_from_file (tmp, NULL);
      x = gdk_pixbuf_get_width (i);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);

      x *= 150;
      x /= 100;

      y *= 150;
      y /= 100;

      i = gdk_pixbuf_new_from_file_at_size (tmp, x, y, NULL);
      break;

    case SIZE_200:
      i = gdk_pixbuf_new_from_file (tmp, NULL);
      x = gdk_pixbuf_get_width (i);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);

      x *= 200;
      x /= 100;

      y *= 200;
      y /= 100;

      i = gdk_pixbuf_new_from_file_at_size (tmp, x, y, NULL);
      break;

    case SIZE_400:
      i = gdk_pixbuf_new_from_file (tmp, NULL);
      x = gdk_pixbuf_get_width (i);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);

      x *= 400;
      x /= 100;

      y *= 400;
      y /= 100;

      i = gdk_pixbuf_new_from_file_at_size (tmp, x, y, NULL);
      break;

    }

  if (!i)
    {
      g_object_unref (i);
      image = gtk_label_new ("Generating image error!");
      gtk_widget_show (image);

      unlink (tmp);

      return image;
    }

  image = gtk_image_new_from_pixbuf (i);
  if (!image)
    {
      g_object_unref (i);
      image = gtk_label_new ("Generating image error!");
      gtk_widget_show (image);

      unlink (tmp);

      return image;
    }

  gtk_widget_show (image);
  g_object_unref (i);

  unlink (tmp);

  return image;
}

void
pdf2ginfo_go_b (GtkWidget * w, GtkWidget *arg)
{
  char *s;

  s = (char *) gtk_entry_get_text (GTK_ENTRY (arg));

  pdf2ginfo_go_page(s, pdf2ginfo_file);
}
	
void
pdf2ginfo_go (GtkWidget * w, char *arg)
{
  char *s;

  s = (char *) gtk_entry_get_text (GTK_ENTRY (w));

  pdf2ginfo_go_page(s, arg);
}

void pdf2ginfo_go_page(char *s, char *arg) {

  pdf2ginfo_page = atoi (s);
  if (pdf2ginfo_page < 1)
    pdf2ginfo_page = 1;
  if (pdf2ginfo_page > pdf2ginfo_pages)
    pdf2ginfo_page = pdf2ginfo_pages;

  gtk_container_remove (GTK_CONTAINER (pdf2ginfo_vp), pdf2ginfo_image);

  pdf2ginfo_image = pdf2ginfo_create_image (arg, pdf2ginfo_page);
  gtk_container_add (GTK_CONTAINER (pdf2ginfo_vp), pdf2ginfo_image);

  pdf2ginfo_check_pages (arg);
}

void
pdf2ginfo_change_next (GtkWidget * w, char *arg)
{
  gtk_container_remove (GTK_CONTAINER (pdf2ginfo_vp), pdf2ginfo_image);
  pdf2ginfo_page++;

  pdf2ginfo_image = pdf2ginfo_create_image (arg, pdf2ginfo_page);
  gtk_container_add (GTK_CONTAINER (pdf2ginfo_vp), pdf2ginfo_image);

  pdf2ginfo_check_pages (arg);
}

void
pdf2ginfo_change_prev (GtkWidget * w, char *arg)
{
  gtk_container_remove (GTK_CONTAINER (pdf2ginfo_vp), pdf2ginfo_image);
  pdf2ginfo_page--;

  pdf2ginfo_image = pdf2ginfo_create_image (arg, pdf2ginfo_page);
  gtk_container_add (GTK_CONTAINER (pdf2ginfo_vp), pdf2ginfo_image);

  pdf2ginfo_check_pages (arg);
}

void
pdf2ginfo_size (GtkWidget * w, int size)
{
  if (pdf2ginfo_scale != size)
    {
      pdf2ginfo_scale = size;

      gtk_container_remove (GTK_CONTAINER (pdf2ginfo_vp), pdf2ginfo_image);

      pdf2ginfo_image =
	pdf2ginfo_create_image (pdf2ginfo_file, pdf2ginfo_page);
      gtk_container_add (GTK_CONTAINER (pdf2ginfo_vp), pdf2ginfo_image);

      pdf2ginfo_check_pages (pdf2ginfo_file);
    }
}

GtkWidget *
pdf2ginfo_popup_create (void)
{
  GtkWidget *menu;
  GtkWidget *size;

  menu = gtk_menu_new ();

  size = gtk_menu_item_new_with_mnemonic ("25%");
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate", G_CALLBACK (pdf2ginfo_size),
		    SIZE_25);

  size = gtk_menu_item_new_with_mnemonic ("50%");
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate", G_CALLBACK (pdf2ginfo_size),
		    SIZE_50);

  size = gtk_menu_item_new_with_mnemonic ("75%");
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate", G_CALLBACK (pdf2ginfo_size),
		    SIZE_75);

  size = gtk_menu_item_new_with_mnemonic ("100%");
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate", G_CALLBACK (pdf2ginfo_size),
		    SIZE_100);

  size = gtk_menu_item_new_with_mnemonic ("125%");
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate", G_CALLBACK (pdf2ginfo_size),
		    SIZE_125);

  size = gtk_menu_item_new_with_mnemonic ("150%");
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate", G_CALLBACK (pdf2ginfo_size),
		    SIZE_150);

  size = gtk_menu_item_new_with_mnemonic ("200%");
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate", G_CALLBACK (pdf2ginfo_size),
		    SIZE_200);

  size = gtk_menu_item_new_with_mnemonic ("400%");
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate", G_CALLBACK (pdf2ginfo_size),
		    SIZE_400);

  size = gtk_menu_item_new ();
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);

  size = gtk_menu_item_new_with_mnemonic ("window size");
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate", G_CALLBACK (pdf2ginfo_size),
		    SIZE_AUTO);

  size = gtk_menu_item_new_with_mnemonic ("window width");
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate", G_CALLBACK (pdf2ginfo_size),
		    SIZE_WIDTH);

  size = gtk_menu_item_new_with_mnemonic ("window height");
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate", G_CALLBACK (pdf2ginfo_size),
		    SIZE_HEIGHT);

  return menu;
}

GtkWidget *
create_pdf (void *arg)
{

  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *sw;
  GtkWidget *entry;
  GtkWidget *button;

  pdf2ginfo_popup_m = pdf2ginfo_popup_create ();

  pdf2ginfo_page = 1;
  pdf2ginfo_file = (char *) arg;
  pdf2ginfo_scale = SIZE_WIDTH;

  vbox = gtk_vbox_new (FALSE, 0);
  hbox = gtk_hbox_new (FALSE, 0);

  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  pdf2ginfo_vp = gtk_viewport_new (NULL, NULL);
  gtk_container_add (GTK_CONTAINER (sw), pdf2ginfo_vp);
  gtk_viewport_set_shadow_type (GTK_VIEWPORT (pdf2ginfo_vp), GTK_SHADOW_NONE);

  gtk_signal_connect_object (GTK_OBJECT (pdf2ginfo_vp), "button_press_event",
			     GTK_SIGNAL_FUNC (pdf2ginfo_popup), NULL);

  pdf2ginfo_image = pdf2ginfo_create_image ((char *) arg, pdf2ginfo_page);

  gtk_container_add (GTK_CONTAINER (pdf2ginfo_vp), pdf2ginfo_image);

  gtk_box_pack_start (GTK_BOX (vbox), sw, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);

  pdf2ginfo_prev = gtk_button_new_with_mnemonic ("_Prev Page");
  gtk_box_pack_start (GTK_BOX (hbox), pdf2ginfo_prev, TRUE, TRUE, 0);
  g_signal_connect ((gpointer) pdf2ginfo_prev, "clicked",
		    G_CALLBACK (pdf2ginfo_change_prev), arg);

  pdf2ginfo_label = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX (hbox), pdf2ginfo_label, TRUE, TRUE, 0);

  entry = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
  g_signal_connect ((gpointer) entry, "activate", G_CALLBACK (pdf2ginfo_go),
		    arg);

  button = gtk_button_new_with_mnemonic ("_Go");
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
  g_signal_connect ((gpointer) button, "clicked", G_CALLBACK (pdf2ginfo_go_b),
		    entry);

  pdf2ginfo_next = gtk_button_new_with_mnemonic ("_Next Page");
  gtk_box_pack_start (GTK_BOX (hbox), pdf2ginfo_next, TRUE, TRUE, 0);
  g_signal_connect ((gpointer) pdf2ginfo_next, "clicked",
		    G_CALLBACK (pdf2ginfo_change_next), arg);

  pdf2ginfo_check_pages ((char *) arg);

  return vbox;
}

int
module_default (void)
{
  return 0;
}

char *
module_name (void)
{
  return "pdf2ginfo";
}

int
module_genre (void)
{
  return GINFO_GENRE_SYSTEM;
}

char **
module_words (void)
{
  char **list;
  DIR *d;
  struct dirent *dir;
  int n = 0;

  if (!(d = opendir (PDF_DIR)))
    {
      puts ("No PDF founded...");
      return NULL;
    }

  while ((dir = readdir (d)) != NULL)
    {
      if (strcmp (dir->d_name + (strlen (dir->d_name) - 4), ".pdf"))
	continue;
      n++;
    }

  if (!(list = (char **) malloc (sizeof (char *) * (n + 2))))
    ginfo_error ("Error: memory.");

  closedir (d);

  if (!(d = opendir (PDF_DIR)))
    {
      free (list);
      puts ("No PDF founded...");
      return NULL;
    }

  n = 0;
  while ((dir = readdir (d)) != NULL)
    {
      if (strcmp (dir->d_name + (strlen (dir->d_name) - 4), ".pdf"))
	continue;

      list[n] = strdup (dir->d_name);
      n++;
    }

  closedir (d);

  list[n] = NULL;

  return list;
}

ginfo_docs *
module_docs (int *n)
{
  void *t;
  DIR *d;
  struct dirent *dir;
  char *buf;
  int i;
  char *path;

  if (!(d = opendir (PDF_DIR)))
    {
      puts ("No PDF founded...");
      return NULL;
    }

  t = ginfo_easy_docs_init ();

  while ((dir = readdir (d)) != NULL)
    {
      if (strcmp (dir->d_name + (strlen (dir->d_name) - 4), ".pdf"))
	continue;

      buf = strdup (dir->d_name);
      for (i = 0; i < strlen (buf); i++)
	if (buf[i] == '.')
	  {
	    buf[i] = 0;
	    break;
	  }

      if (!
	  (path =
	   (char *) malloc (sizeof (char) *
			    (strlen (PDF_DIR) + strlen (dir->d_name) + 2))))
	ginfo_error ("Memory!");
      sprintf (path, "%s/%s", PDF_DIR, dir->d_name);

      ginfo_easy_docs_add (t, strdup (buf), create_pdf, path, NULL);
      free (buf);
    }

  closedir (d);

  return ginfo_easy_docs_close (t, n);
}
