/* gInfo - Copyright (C) 2004-5 bakunin - Andrea Marchesini 
 *                                        <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "ginfo.h"

typedef struct _search_
{
  module *m;
  struct _search_ *next;
} search;

GtkWidget *search_window = NULL;
static GtkWidget *search_result_list;
static GtkTreeSelection *search_selection;
static search *search_result_index = NULL;


search *search_module (char *, module *);
int search_algoritm_string (char *, module *);
void search_close (GtkWidget *, gpointer);
void search_init (search *);
void search_free_result (void);
void search_no_result (void);
void search_set_result (search *);
char *search_set_result_string (module *);
gint search_select (GtkWidget *, GdkEventButton *);
void search_show_result (GtkWidget *, gpointer);
int search_select_check (void);

/* Do you want search something ? */
void
search_char (char *a)
{
  module *m;
  search *t;

  m = module_head;

  /* For every modules... */
  while (m)
    {

      /* ...search your words */
      if ((t = search_module (a, m)))
	{
	  /* and I create a list */
	  t->next = search_result_index;
	  search_result_index = t;
	}

      m = m->next;
    }

  /* I show the list */
  if (search_result_index)
    search_init (search_result_index);
  else
    search_no_result ();
}

/* For a single module */
search *
search_module (char *a, module * m)
{
  char **str, **k;
  search *t;

  /* Split the string to array */
  str = split_string (a);
  k = str;

  while (str && *str)
    {
      /* Compare... like strstr */
      if (search_algoritm_string (*str, m))
	{
	  if (!(t = (search *) malloc (sizeof (search))))
	    fatal ("Error: memory.");
	  t->m = m;

	  return t;
	}

      str++;
    }

  free (k);

  return NULL;
}

/* Only this... */
int
search_algoritm_string (char *a, module * m)
{
  char **w;
  char *t;
  int l1, l2;

  w = m->words;

  while (w && *w)
    {

      t = *w;
      l2 = strlen (a);
      l1 = strlen (t);

      while (l2 <= l1)
	{

	  l1 = strlen (t);

	  if (!strncasecmp (a, t, l2))
	    return 1;

	  t++;
	}

      w++;
    }

  return 0;
}

/* This function closes the search window */
void
search_close (GtkWidget * w, gpointer dummy)
{
  gtk_widget_destroy (search_window);

  search_free_result ();

  search_window = NULL;
}

/* Free memory */
void
search_free_result ()
{
  search *t;

  while (search_result_index)
    {
      t = search_result_index;

      search_result_index = search_result_index->next;

      free (t);
    }
}

/* No result window */
void
search_no_result (void)
{

  if (search_window)
    return;

  search_window =
    gtk_message_dialog_new (GTK_WINDOW (mainwin),
			    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			    GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
			    "Sorry, no result");

  gtk_dialog_run (GTK_DIALOG (search_window));
  gtk_widget_destroy (search_window);

  search_window = NULL;
}

/* Search window */
void
search_init (search * t)
{
  GtkWidget *frame;
  GtkWidget *box;
  GtkWidget *hbox;
  GtkWidget *scrolledwindow;
  GtkListStore *model;
  GtkCellRenderer *renderer;
  GtkWidget *button;

  char buf[255];

  if (search_window)
    return;

  snprintf (buf, 255, "%s %s - search", PACKAGE, VERSION);

  search_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (search_window), buf);
  gtk_window_set_position (GTK_WINDOW (search_window), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (search_window), TRUE);

  g_signal_connect ((gpointer) search_window, "destroy",
		    G_CALLBACK (search_close), NULL);

  frame = gtk_frame_new ("Results");
  gtk_widget_show (frame);
  gtk_container_add (GTK_CONTAINER (search_window), frame);

  box = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (box);
  gtk_container_add (GTK_CONTAINER (frame), box);

  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow);
  gtk_box_pack_start (GTK_BOX (box), scrolledwindow, TRUE, TRUE, 0);
  gtk_widget_set_size_request (scrolledwindow, 250, -1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  model = gtk_list_store_new (1, G_TYPE_STRING);
  search_result_list = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
  search_selection =
    gtk_tree_view_get_selection (GTK_TREE_VIEW (search_result_list));
  gtk_tree_selection_set_mode (search_selection, GTK_SELECTION_SINGLE);
  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (search_result_list), TRUE);
  gtk_tree_view_set_search_column (GTK_TREE_VIEW (search_result_list), 0);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW
					       (search_result_list), -1,
					       "Results", renderer, "text", 0,
					       NULL);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (search_result_list),
				     FALSE);

  gtk_widget_show (search_result_list);
  g_object_unref (model);

  gtk_container_add (GTK_CONTAINER (scrolledwindow), search_result_list);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);

  button = gtk_button_new_with_mnemonic ("_Show");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);

  g_signal_connect ((gpointer) button, "clicked",
		    G_CALLBACK (search_show_result), NULL);

  button = gtk_button_new_with_mnemonic ("_Cancel");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);

  g_signal_connect ((gpointer) button, "clicked", G_CALLBACK (search_close),
		    NULL);

  search_set_result (t);

  gtk_signal_connect_object (GTK_OBJECT (search_result_list),
			     "button_press_event",
			     GTK_SIGNAL_FUNC (search_select), NULL);

  gtk_widget_show (search_window);
}

/* I write the result in the treeview */
void
search_set_result (search * t)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  char *buf;

  model = gtk_tree_view_get_model (GTK_TREE_VIEW (search_result_list));

  while (t)
    {
      gtk_list_store_append (GTK_LIST_STORE (model), &iter);

      buf = search_set_result_string (t->m);
      gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, buf, -1);
      free (buf);

      t = t->next;
    }
}

char *
search_set_result_string (module * m)
{
  char *k;
  int n;

  if ((m->genre & GINFO_GENRE_SYSTEM))
    {
      n = strlen (m->name) + strlen ("System: ") + 1;
      if (!(k = (char *) malloc (sizeof (char) * n)))
	fatal ("Error: memory.");

      snprintf (k, n, "System: %s", m->name);
      return k;
    }

  if ((m->genre & GINFO_GENRE_NETWORK))
    {
      n = strlen (m->name) + strlen ("Network: ") + 1;
      if (!(k = (char *) malloc (sizeof (char) * n)))
	fatal ("Error: memory.");

      snprintf (k, n, "Network: %s", m->name);
      return k;
    }

  if ((m->genre & GINFO_GENRE_AUDIO))
    {
      n = strlen (m->name) + strlen ("Audio: ") + 1;
      if (!(k = (char *) malloc (sizeof (char) * n)))
	fatal ("Error: memory.");

      snprintf (k, n, "Audio: %s", m->name);
      return k;
    }

  if ((m->genre & GINFO_GENRE_VIDEO))
    {
      n = strlen (m->name) + strlen ("Video: ") + 1;
      if (!(k = (char *) malloc (sizeof (char) * n)))
	fatal ("Error: memory.");

      snprintf (k, n, "Video: %s", m->name);
      return k;
    }

  if ((m->genre & GINFO_GENRE_STREAM))
    {
      n = strlen (m->name) + strlen ("Stream: ") + 1;
      if (!(k = (char *) malloc (sizeof (char) * n)))
	fatal ("Error: memory.");

      snprintf (k, n, "Stream: %s", m->name);
      return k;
    }

  if ((m->genre & GINFO_GENRE_OFFICE))
    {
      n = strlen (m->name) + strlen ("Office: ") + 1;
      if (!(k = (char *) malloc (sizeof (char) * n)))
	fatal ("Error: memory.");

      snprintf (k, n, "Office: %s", m->name);
      return k;
    }


  if ((m->genre & GINFO_GENRE_CODING))
    {
      n = strlen (m->name) + strlen ("Coding: ") + 1;
      if (!(k = (char *) malloc (sizeof (char) * n)))
	fatal ("Error: memory.");

      snprintf (k, n, "Coding: %s", m->name);
      return k;
    }

  n = strlen (m->name) + strlen ("Other: ") + 1;
  if (!(k = (char *) malloc (sizeof (char) * n)))
    fatal ("Error: memory.");

  snprintf (k, n, "Other: %s", m->name);
  return k;

}

/* I search your choise */
gint
search_select (GtkWidget * w, GdkEventButton * event)
{
  if (event->type == GDK_2BUTTON_PRESS)
    {
      if (search_select_check ());
      return FALSE;
    }

  return FALSE;
}

void
search_show_result (GtkWidget * w, gpointer dummy)
{
  search_select_check ();
}

int
search_select_check (void)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreePath *path;
  int n;
  search *s = search_result_index;

  if (gtk_tree_selection_get_selected (search_selection, &model, &iter))
    {
      path = gtk_tree_model_get_path (model, &iter);

      n = gtk_tree_path_get_indices (path)[0];

      gtk_tree_path_free (path);

      while (n)
	{
	  s = s->next;
	  n--;
	}

      tutor_container (s->m->docs->func, s->m->docs->arg);

      gtk_widget_destroy (search_window);

      return 1;
    }

  return 0;
}

/* EOF */
