/* gInfo - Copyright (C) 2004-5 bakunin - Andrea Marchesini 
 *                                        <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "ginfo.h"

int socket_closed;

int socket_open (void);
void socket_close (int);
void socket_start_module (char *);

/* This function is a separated thread that listens on a unix socket. 
 * Read the id of a documentation and show it */
void *
socket_admin (void *arg)
{
  int fd, c, ret, i;
  char buf[1024];
  struct sockaddr_un incoming;
  size_t size_un;
  fd_set set;
  struct timeval tv;

  if ((fd = socket_open ()) < 0)
    {
      msg ("I can't open the unix socket!");
      pthread_exit (NULL);
    }

  while (!socket_closed)
    {
      FD_ZERO (&set);
      FD_SET (fd, &set);

      tv.tv_sec = 0;
      tv.tv_usec = 100000;

      size_un = sizeof (struct sockaddr_un);

      if ((select (fd + 1, &set, NULL, NULL, &tv) <= 0)
	  || ((c = accept (fd, (struct sockaddr *) &incoming, &size_un)) ==
	      -1))
	continue;

      if ((ret = read (c, buf, sizeof (buf))) < 0)
	{
	  msg ("Unix Socket read error!");
	  continue;
	  break;
	}

      buf[ret - 1] = 0;

      for (i = 0; i < ret; i++)
	{
	  if (buf[i] == '\n')
	    {
	      buf[i] = 0;
	      break;
	    }
	}

      if (strlen (buf))
	socket_start_module (buf);

      close (c);
    }

  socket_close (fd);

  pthread_exit (NULL);
}

/* the unix socket */
int
socket_open (void)
{
  struct sockaddr_un saddr;
  int fd;

  if ((fd = socket (AF_UNIX, SOCK_STREAM, 0)) < 0)
    return -1;

  saddr.sun_family = AF_UNIX;
  snprintf (saddr.sun_path, 108, "%s/ginfo.sock", GINFO_TMP_DIR);
  if ((unlink (saddr.sun_path) == -1) && errno != ENOENT)
    {
      msg ("I can't remove %s files.", saddr.sun_path);
      return -1;
    }

  if (bind (fd, (struct sockaddr *) &saddr, sizeof (saddr)) < 0)
    {
      msg ("I can't bind the unix socket: %s!", saddr.sun_path);
      close (fd);
      return -1;
    }

  if (listen (fd, 5) < 0)
    {
      msg ("I can't set the listen to unix socket!");
      close (fd);
      return -1;
    }

  return fd;
}

void
socket_close (int fd)
{
  char buf[108];

  close (fd);

  snprintf (buf, 108, "%s/ginfo.sock", GINFO_TMP_DIR);
  unlink(buf);
}

/* This function alloc in module_next_popup the id 
 * The principal thread read this string (with a timeout function) and
 * show the module. I can't open the tutor win in this thread because
 * gtk_main runs in the other thread. */
void
socket_start_module (char *id)
{
  module_next_popup = strdup (id);
}

/* This is the first thread that, with a timer, run this funcion */
int
socket_timeout_check (gpointer dummy)
{

  /* Check new data from socket */
  if (module_next_popup)
    {
      doc *d = tooltips_find_id (module_next_popup);

      free (module_next_popup);
      module_next_popup = NULL;

      if (d)
	tutor_container (d->func, d->arg);
    }

  return 1;
}

/* EOF */
